From c2a471bbf093d559f1a78d5d23ecabe0a9c0ead8 Mon Sep 17 00:00:00 2001
From: Henrik Gramner <henrik@gramner.com>
Date: Wed, 10 Jun 2020 00:14:38 +0200
Subject: [PATCH] Add support for specifying an initial idr_pic_id

---
 autocomplete.c    |  1 +
 common/base.c     |  3 +++
 encoder/encoder.c | 10 ++++------
 x264.c            |  2 ++
 x264.h            |  3 ++-
 5 files changed, 12 insertions(+), 7 deletions(-)

diff --git a/autocomplete.c b/autocomplete.c
index 516d9f21..3ff19a38 100644
--- a/autocomplete.c
+++ b/autocomplete.c
@@ -110,6 +110,7 @@ static const char * const opts_nosuggest[] =
     "--deadzone-intra",
     "--fps",
     "--frames",
+    "--init-idr-pic-id",
     "--input-depth",
     "--input-res",
     "--ipratio",
diff --git a/common/base.c b/common/base.c
index abca9144..12316f93 100644
--- a/common/base.c
+++ b/common/base.c
@@ -415,6 +415,7 @@ REALIGN_STACK void x264_param_default( x264_param_t *param )
     param->b_pic_struct = 0;
     param->b_fake_interlaced = 0;
     param->i_frame_packing = -1;
+    param->i_init_idr_pic_id = 0;
     param->i_alternative_transfer = 2; /* undef */
     param->b_opencl = 0;
     param->i_opencl_device = 0;
@@ -1289,6 +1290,8 @@ REALIGN_STACK int x264_param_parse( x264_param_t *p, const char *name, const cha
         p->i_frame_packing = atoi(value);
     OPT("stitchable")
         p->b_stitchable = atobool(value);
+    OPT("init-idr-pic-id")
+        p->i_init_idr_pic_id = atoi(value);
     OPT("opencl")
         p->b_opencl = atobool( value );
     OPT("opencl-clbin")
diff --git a/encoder/encoder.c b/encoder/encoder.c
index 968e2735..966ff15c 100644
--- a/encoder/encoder.c
+++ b/encoder/encoder.c
@@ -805,6 +805,7 @@ static int validate_parameters( x264_t *h, int b_open )
         h->param.i_nal_hrd = X264_NAL_HRD_NONE;
         h->param.b_deblocking_filter = 0;
         h->param.b_stitchable = 1;
+        h->param.i_init_idr_pic_id = 5;
         h->param.b_pic_struct = 0;
         h->param.analyse.b_transform_8x8 = 1;
         h->param.analyse.intra = X264_ANALYSE_I8x8;
@@ -1293,6 +1294,7 @@ static int validate_parameters( x264_t *h, int b_open )
         h->param.rc.f_complexity_blur = 0;
 
     h->param.i_sps_id &= 31;
+    h->param.i_init_idr_pic_id &= 0xffff;
 
     h->param.i_nal_hrd = x264_clip3( h->param.i_nal_hrd, X264_NAL_HRD_NONE, X264_NAL_HRD_CBR );
 
@@ -1492,11 +1494,7 @@ x264_t *x264_encoder_open( x264_param_t *param )
     /* Init x264_t */
     h->i_frame = -1;
     h->i_frame_num = 0;
-
-    if( h->param.i_avcintra_class )
-        h->i_idr_pic_id = 5;
-    else
-        h->i_idr_pic_id = 0;
+    h->i_idr_pic_id = h->param.i_init_idr_pic_id;
 
     if( (uint64_t)h->param.i_timebase_den * 2 > UINT32_MAX )
     {
@@ -2555,7 +2553,7 @@ static inline void slice_init( x264_t *h, int i_nal_type, int i_global_qp )
             }
         }
         else
-            h->i_idr_pic_id ^= 1;
+            h->i_idr_pic_id = !h->i_idr_pic_id;
     }
     else
     {
diff --git a/x264.c b/x264.c
index 1d6334ee..6039f5a3 100644
--- a/x264.c
+++ b/x264.c
@@ -938,6 +938,7 @@ static void help( x264_param_t *defaults, int longhelp )
         "                                  - %s\n", x264_avcintra_flavor_names[0], stringify_names( buf, x264_avcintra_flavor_names ) );
     H1( "      --stitchable            Don't optimize headers based on video content\n"
         "                              Ensures ability to recombine a segmented encode\n" );
+    H2( "      --init-idr-pic-id <integer> Initial idr_pic_id [%d]\n", defaults->i_init_idr_pic_id );
     H1( "\n" );
     H1( "  -v, --verbose               Print stats for each frame\n" );
     H1( "      --no-progress           Don't show the progress indicator while encoding\n" );
@@ -1187,6 +1188,7 @@ static struct option long_options[] =
     { "output-csp",  required_argument, NULL, OPT_OUTPUT_CSP },
     { "input-range", required_argument, NULL, OPT_INPUT_RANGE },
     { "stitchable",        no_argument, NULL, 0 },
+    { "init-idr-pic-id", required_argument, NULL, 0 },
     { "filler",            no_argument, NULL, 0 },
     {0, 0, 0, 0}
 };
diff --git a/x264.h b/x264.h
index 3a50f1c8..88a1b6a6 100644
--- a/x264.h
+++ b/x264.h
@@ -45,7 +45,7 @@ extern "C" {
 
 #include "x264_config.h"
 
-#define X264_BUILD 160
+#define X264_BUILD 161
 
 #ifdef _WIN32
 #   define X264_DLL_IMPORT __declspec(dllimport)
@@ -528,6 +528,7 @@ typedef struct x264_param_t
      * each part, and stitching them back together will result in identical SPS/PPS. This is necessary for stitching
      * with container formats that don't allow multiple SPS/PPS. */
     int b_stitchable;
+    int i_init_idr_pic_id;   /* Initial idr_pic_id. The id will alternate between 0 and 1 afterwards. */
 
     int b_opencl;            /* use OpenCL when available */
     int i_opencl_device;     /* specify count of GPU devices to skip, for CLI users */
-- 
2.20.1

